import * as http from "node:http";
import * as https from "node:https";
import net from "node:net";
import { HttpProxyAgent, HttpsProxyAgent } from "hpagent";
import config from "@/config/index.js";
import IPCIDR from "ip-cidr";
import PrivateIp from "private-ip";

declare module 'node:http' {
	interface Agent {
		createConnection(options: net.NetConnectOpts, callback?: (err: unknown, stream: net.Socket) => void): net.Socket;
	}
}

function isPrivateIp(ip: string): boolean {
	for (const net of config.allowedPrivateNetworks || []) {
		const cidr = new IPCIDR(net);
		if (cidr.contains(ip)) {
			return false;
		}
	}

	return PrivateIp(ip);
}

function checkConnection(socket: net.Socket) {
	const address = socket.remoteAddress;
	if (process.env.NODE_ENV === 'production') {
		if (address && IPCIDR.isValidAddress(address) && isPrivateIp(address)) {
			socket.destroy(new Error(`Blocked address: ${address}`));
		}
	}
}

export class CheckedHttpAgent extends http.Agent {
	createConnection(options: net.NetConnectOpts, callback?: (err: unknown, stream: net.Socket) => void): net.Socket {
		const socket = super.createConnection(options, callback).on('connect', () => { checkConnection(socket) });
		return socket;
	}
}

export class CheckedHttpsAgent extends https.Agent {
	createConnection(options: net.NetConnectOpts, callback?: (err: unknown, stream: net.Socket) => void): net.Socket {
		const socket = super.createConnection(options, callback).on('connect', () => { checkConnection(socket) });
		return socket;
	}
}
export class CheckedHttpProxyAgent extends HttpProxyAgent {
	createConnection(options: net.NetConnectOpts, callback?: (err: unknown, stream: net.Socket) => void): net.Socket {
		const socket = super.createConnection(options, callback).on('connect', () => { checkConnection(socket) });
		return socket;
	}
}

export class CheckedHttpsProxyAgent extends HttpsProxyAgent {
	createConnection(options: net.NetConnectOpts, callback?: (err: unknown, stream: net.Socket) => void): net.Socket {
		const socket = super.createConnection(options, callback).on('connect', () => { checkConnection(socket) });
		return socket;
	}
}
